// Handle information about conferences with unread texts.
//
// $Id: unread-confs.h,v 1.10 1994/07/16 17:47:43 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef UNREAD_CONFS__
#define UNREAD_CONFS__

#include <iostream.h>
#include <DLList.h>
#include "array.h"
#include "session.h"
#include "read-order.h"

class get_unread_confs_question;

typedef struct {
    Conf_no   conf;
    long      min_unread;	// At least this many unread texts.
    long      max_unread;	// At most this many unread texts.
				// -1 if no upper limit can be computed yet.
} Unread_info;


// Which conferences do we have unread texts in?

class Unread_confs {
  public:
    Unread_confs(Session *sess, Pers_no the_reader);
    Unread_confs(const Unread_confs &);
    ~Unread_confs();

    // This method can be used to require a certain amount of
    // knowledge about what is unread. Prefetching is done according
    // to these criteria:
    //  + at least TEXTS texts should be known
    //  + we should know about at least TEXT_IN_CONFS unread texts in
    //    at least CONFS conferences.
    //  + at most MAX_QUESTION questions are sent to the server
    //    simultaneously.
    //  + at least MIN_QUESTION questions should be pending even if we
    //    know that much (so that we eventually end up knowing about
    //    all unread texts).
    // Returns the number of pending questions caused by this
    // Unread_confs instance.
    int                  prefetch_map(int texts = 20,
				      int text_in_confs = 4,
				      int confs = 3,
				      int max_question = 3,
				      int min_question = 1);

    // Like prefetch_map, but prefetch _everything_ that is needed by
    // nilkom to display the texts.
    int			prefetch(int texts = 15,
				 int text_in_confs = 2,
				 int confs = 2,
				 int max_question = 3,
				 int min_question = 0);

    // goto_next_conf() will perform a pepsi(). It will return 0 if
    // there has been a fatal error (such as a lost connection).
    Conf_no              goto_next_conf();
    Status		 goto_conf(Conf_no);

    // Returns PROMPT_ERROR if there is a fatal error.
    Prompt               next_prompt();

    // Returns 0 if there is a fatal error, or if there are no more
    // texts to be read.
    Text_no              next_text(Bool remove=FALSE);
    Conf_no              current_conf();
    Array<Unread_info>   unread_texts();
    long                 min_unread_texts_in_current();
    long                 max_unread_texts_in_current();

    // Add a new conference (this should be done when the user wants
    // to join a conference). If LAST is true, append it to the list
    // of unread conferences, otherwise prepend it.
    void                 add_conference(Conf_no, Bool last);

  private:
    const Unread_confs & operator = (const Unread_confs &); // NYI

    Session              * session;
    Conferences          & conferences;
    Persons		 & persons;
    Maps		 & maps;
    Texts		 & texts;

    DLList<Read_order *>   unread_confs;
    // Since the Read_order objects contains state that is useful even
    // when we have read all texts, we save them here. +++ The state
    // should be removed from them. The state that is useful is the
    // membership, and we really ought to have a cache for them.
    DLList<Read_order *>   read_confs;
    Pers_no		   reader;
    Conf_no		   current_working_conf;

    get_unread_confs_question *ucq;
    Bool unread_confs_has_been_fetched;
};
#endif  // UNREAD_CONFS__
