// A simple resemblance of the more command.
//
// $Id: paginator.cc,v 1.4 1994/07/16 16:39:06 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// paginator.cc

#include <assert.h>

#include "paginator.h"

Paginator::Paginator(ostream &screen)
     :sout(screen)
{
  n_rows = 12;
  n_cols = 80;
  cursor_row = 0;
  cursor_col = 0;
}

Paginator::~Paginator()
{
}

int 
Paginator::column() const
{
  return cursor_col;
}

int 
Paginator::rows_left() const
{
  // You cannot write on the last row - it is reserved for the prompt.
  return n_rows - cursor_row - 1;
}

int 
Paginator::columns() const
{
  return n_cols;
}

int 
Paginator::rows() const
{
  return n_rows;
}

Paginator::Screenstatus 
Paginator::print(char c, Screenarea area)
{
  assert(cursor_col >= 0);
  assert(cursor_col < n_cols);
  assert(cursor_row >= 0);
  assert(cursor_row < n_rows);

  unsigned char uc = c;

  // Handle special characters.
  switch(uc)
    {
    case '\b':
      //Backspace is special - but only in the prompt area.
      if (area == PROMPT)
	{
	  if (cursor_col > 0)
	    {
	      cursor_col--;
	      sout << uc;
	    }
	  return SPACE_LEFT;
	}
      break;
    case '\r':
      sout << uc;
      cursor_col = 0;
      return SPACE_LEFT;
      
    case '\n':
      if (cursor_row == n_rows - 1)
	return FULL;

      sout << '\r' << '\n';
      cursor_col = 0;
      cursor_row++;
      return SPACE_LEFT;
      
    case '\t':
      {
	Screenstatus ret;

	while ((ret = print(' ', area)) == SPACE_LEFT
	       && cursor_col % 8 != 0)
	  ;
	return ret;
      }
    }

  // FIXME: use locale stuff to determine if it is printable or not
  if (uc < ' ' || (uc >= 0x80 && uc < ' ' + 0x80))
    {
      // Control character.
      // FIXME: configurable printing would be nice.

      enum {GLYPH_SIZE = 4};
      char buffer[GLYPH_SIZE+1];
      char *end;
      sprintf(buffer, "\\%03o", uc);

      if (area == PROMPT)
	{
	  if (cursor_row == n_rows - 1 && cursor_col >= n_cols - GLYPH_SIZE)
	    return FULL;
	}
      else
	{
	  if ((cursor_row == n_rows - 2 && cursor_col > n_cols - GLYPH_SIZE)
	      || cursor_row == n_rows - 1)
	    return FULL;
	}

      print(buffer, &end, area);
      assert(end == buffer+GLYPH_SIZE);
      assert(*end == '\0');
      return SPACE_LEFT;
    }

  // Print normal characters.

  if (cursor_row == n_rows - 1 
      && (area == USER || cursor_col == n_cols - 1))
    return FULL;

  sout << uc;
  if (++cursor_col >= n_cols)
    {
      cursor_col = 0;
      cursor_row++;
      
      assert(cursor_row < n_rows);
    }
  return SPACE_LEFT;
}

Paginator::Screenstatus 
Paginator::dprint(LyStr &str, Screenarea area)
{
  while (str.strlen() > 0)
    if (print(str[0], area) == FULL)
      return FULL;
    else
      str.remove_first(1);

  return SPACE_LEFT;
}

Paginator::Screenstatus
Paginator::print(const char *str,
		 const char **remains,
		 Screenarea area)
{
  assert(str != NULL);

  for (const char *p = str; *p != '\0'; p++)
    if (print(*p, area) == FULL)
      {
	if (remains != NULL)
	  *remains = p;
	return FULL;
      }
  
  if (remains != NULL)
    *remains = p;
  return SPACE_LEFT;
}


void 
Paginator::clearline()
{
  sout << '\r';
  for (int ix = 0; ix < n_cols - 1; ix++)
    sout << ' ';
  sout << '\r';
  cursor_col = 0;
}

void 
Paginator::acknowledge_lines()
{
  // FIXME: The following zero should be a configuration option,
  // so that it is possible to say "I want 3 lines of context".
  cursor_row = 0;
}

// eof
