// paginator.h - Low-level *more* prompt handling
// 
// $Id: paginator.h,v 1.3 1994/07/16 16:39:10 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef PAGINATOR_H
#define PAGINATOR_H

#include <iostream.h>

#include "LyStr.h"
#include "misc.h"

// The Paginator class is responsible for sending output to a screen.
// It knows the width and height of the screen, and makes sure that
// data never scrolls past the user before he has had a chance to see
// it.
//
// This class is a little bit tricky to use.  The interface could be
// simpler.  It is the way it is because it is intended to be used in
// conjunction with TCL the way it is used in nilkom.
//
class Paginator {
public:
    // Screen width, height and type will be fetched from the
    // environment.  Paginator::Paginator will (FIXME in a future
    // version) catch SIGWINCH.
    Paginator(ostream &screen=cout);
    ~Paginator();
    
    // Return current column number (0 is leftmost column).
    int column() const;
    // Return number of lines left to write on before a *more* prompt
    // is needed.
    int rows_left() const;
    
    // Return current screen dimensions.
    int columns() const;
    int rows() const;
    
    enum Screenstatus {
	FULL,
	SPACE_LEFT
    };
    enum Screenarea {
        USER,
        PROMPT
    };
    
    // Write character ``c'' if there is room for it.
    // Nothing has been written if FULL is returned.
    Screenstatus print(char c, Screenarea area=USER);

    // Write as much as possible of the string ``str'' to the screen.
    // Remove whatever is written to the screen and retain the rest
    // in ``str''.
    // If ``area'' is PROMPT, allow writing on the last line.
    Screenstatus dprint(LyStr &str, Screenarea area=USER);

    // Write as much as possible of the NUL-terminated string ``str''
    // to the screen.
    // Set ``remains'' to point to whatever didn't fit on the screen.
    // (``remains'' will point into ``str''.  If it is possible to
    // write everything, ``remains'' will point to the terminating NUL).
    // If ``area'' is PROMPT, allow writing on the last line.
    Screenstatus print(const char *str, // in
		       const char **remains, // out
		       Screenarea area=USER);

    // Clear the current line and move the cursor to the left-most column.
    void clearline();

    // The user has seen everything which is printed, so no prompt is
    // needed for another screenfull.    
    void acknowledge_lines();
    
private:
    // The screen.
    ostream &sout;
    
    // Current screen size.
    int n_rows;
    int n_cols;

    // Current cursor position.  Note: ``cursor_row'' is updated as if
    // the screen was cleared everytime a *more* prompt is removed,
    // even though the normal mode is to use scrolling.
    int cursor_row;
    int cursor_col;

    // These are not, and should not currently be, implemented.
    Paginator(const Paginator&);
    Paginator &operator =(const Paginator&);
};

#endif
