// Implementation of the TCL commands used in nilkom_print.tcl
//
// $Id: userio.cc,v 1.4 1994/07/16 16:40:18 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <strstream.h>
#include <assert.h>
#include <stdlib.h>

#include "tcl-support.h"
#include "tcl-main.h"
#include "connection.h"
#include "userio.h"
#include "paginator.h"
#include "LyStr.h"

static LyStr pending_output;
static Paginator pout;

extern "C" int
tcl_print(ClientData clientData,
	  Tcl_Interp *interp, 
	  int argc, 
	  char *argv[])
{
    if (argc != 2)
    {
	Tcl_SetResult(interp, "One argument expected", TCL_STATIC);
	Tcl_SetErrorCode(interp, "KOM", "BADARGS", (char *)NULL);
	return TCL_ERROR;
    }

    pending_output << argv[1];
    pout.dprint(pending_output);
    return TCL_OK;
}

enum Kom_page_args {
    PAGE_PENDING=1, PAGE_PROMPT, PAGE_PRINT, PAGE_NEXT, PAGE_DISCARD, 
    PAGE_SYNC, PAGE_ACK
    };

static Tcl_Args kom_page_options[] = {
    {"pending",      PAGE_PENDING, 0, 0, {}},
    {"prompt",       PAGE_PROMPT,  1, 0, {TCLARG_STRING}},
    {"prompt_print", PAGE_PRINT,   1, 0, {TCLARG_STRING}},
    {"next",         PAGE_NEXT,    0, 0, {}},
    {"discard",      PAGE_DISCARD, 0, 0, {}},
    {"sync",         PAGE_SYNC,    0, 0, {}},
    {"acknowledge",  PAGE_ACK,     0, 0, {}}
};

extern "C" int
tcl_page(ClientData clientData,
	 Tcl_Interp *interp, 
	 int argc, 
	 char *argv[])
{
  Parse_tcl_args args(interp, kom_page_options, argc, argv);
  switch (args.option())
    {
    case PAGE_PROMPT:
      pout.clearline();
      pout.print(argv[2], NULL, Paginator::PROMPT);
      break;
      
    case PAGE_PRINT:
      pout.print(argv[2], NULL, Paginator::PROMPT);
      break;
      
    case PAGE_NEXT:
      pout.clearline();
      pout.acknowledge_lines();
      pout.dprint(pending_output);
      break;

    case PAGE_PENDING:
      {
	ostrstream res(interp->result, TCL_RESULT_SIZE);
	res << (pending_output.strlen() > 0 ? 1 : 0) << '\0';
      }
      break;
      
    case PAGE_DISCARD:
      pending_output.clear();
      break;

    case PAGE_SYNC:
      cout.flush();
      break;

    case PAGE_ACK:
      pout.acknowledge_lines();
      break;

    default:
      assert(args.option() < 0);
      return TCL_ERROR;
      
    };
  return TCL_OK;
}

extern "C" int
tcl_getkey(ClientData clientData,
	   Tcl_Interp *interp, 
	   int argc, 
	   char *argv[])
{
  if (argc != 1)
    {
      Tcl_SetResult(interp, "No arguments expected", TCL_STATIC);
      Tcl_SetErrorCode(interp, "KOM", "BADARGS", (char *)NULL);
      return TCL_ERROR;
    }

  cout.flush();

  while (1)
    {
      if (unread_command_string.strlen() > 0)
	{
	  ostrstream    res(interp->result, TCL_RESULT_SIZE);
	  res << int(unread_command_string[0]) << '\0';
	  unread_command_string.remove_first(1);

	  return TCL_OK;
	}

      IscEvent *ev = Connection::get_event(1000);
      if (ev == &Connection::error_event)
	{
	  Tcl_SetResult(interp, "Connection lost", TCL_STATIC);
	  Tcl_SetErrorCode(interp, "KOM", "LOST_CONNECTION", (char *)NULL);
	  return TCL_ERROR;
	}

      if (ev != NULL && ev->event != ISC_EVENT_TIMEOUT)
	{
	  assert (ev->session == stdincb);
	  switch(ev->event)
	    {
	    case ISC_EVENT_ERROR:
	      abort();
	    case ISC_EVENT_MESSAGE:
	      unread_command_string << LyStr(ev->msg->buffer,
					     ev->msg->length);
	      break;
	    default:
	      abort();
	    }
	}
      else if (ev != NULL)
	{
	  assert(ev->event == ISC_EVENT_TIMEOUT);
	  char buf[10];
	  sprintf(buf, "timeout");
	  int res = Tcl_Eval(interp, buf);
	  if (res != TCL_OK)
 	    return res;

	  if (interp->result[0] != '0' || interp->result[1] != '\0')
	    {
	      Tcl_SetResult(interp, "Call for termination", TCL_STATIC);
	      Tcl_SetErrorCode(interp, "KOM", "TERMINATE");
	      return TCL_ERROR;
	    }
	}

      if (ev != NULL)
	isc_dispose(ev);
    }
}

void define_userio_commands(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "nilkom_print",   tcl_print,  NULL, NULL);
  Tcl_CreateCommand(interp, "nilkom_page",    tcl_page,   NULL, NULL);
  Tcl_CreateCommand(interp, "nilkom_get_key", tcl_getkey, NULL, NULL);
}
